#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  aliplat.py
@Date    :  2021/06/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import json
import math
import os
from xml.dom.minidom import parseString

import aigpy
import requests
from aigpy.progressHelper import ProgressTool

from b2a.platformImp import *

requests.packages.urllib3.disable_warnings()


class AliUploadLink(object):
    def __init__(self, jsonData, localFilePath, remoteFilePath):
        self.list = jsonData.get('part_info_list', [])
        self.fileId = jsonData.get('file_id')
        self.uploadId = jsonData.get('upload_id')
        self.needUpload = True
        self.localFilePath = localFilePath
        self.remoteFilePath = remoteFilePath

        if 'rapid_upload' in jsonData and jsonData['rapid_upload']:
            self.needUpload = False
        if 'exist' in jsonData and jsonData['exist']:
            self.needUpload = False


class AliKey(object):
    def __init__(self):
        super().__init__()
        self.chunkSize = 10485760
        self.driveId = ''
        self.refreshToken = ''
        self.headers = None
        self.pathIds = {'/': 'root'}

    def login(self, refreshToken: str) -> bool:
        try:
            data = {"refresh_token": refreshToken}
            post_json = requests.post('https://websv.aliyundrive.com/token/refresh',
                                      data=json.dumps(data),
                                      headers={'content-type': 'application/json;charset=UTF-8'},
                                      verify=False).json()
            self.headers = {
                'authorization': post_json['access_token'],
                'content-type': 'application/json;charset=UTF-8'
            }
            self.driveId = post_json['default_drive_id']
            self.refreshToken = post_json['refresh_token']
            return True
        except Exception as e:
            return False

    def __getPathId__(self, remotePath: str):
        if len(remotePath) <= 0:
            remotePath = '/'
        if remotePath in self.pathIds:
            return self.pathIds[remotePath]

        parent = ''
        paths = remotePath.split('/')
        for item in paths:
            key = parent + '/' + item
            key = key.replace('//', '/')
            if key not in self.pathIds:
                res = self.list(parent)
                if len(res) <= 0:
                    return None
                if key not in self.pathIds:
                    return None
            parent = key

        return self.pathIds[remotePath]

    def __updatePathId__(self, path: str, sid: str):
        self.pathIds[path] = sid

    def list(self, remotePath: str) -> List[FileAttr]:
        remotePath = remotePath.rstrip('/')
        sid = self.__getPathId__(remotePath)
        if not sid:
            return []

        try:
            requests_data = {"drive_id": self.driveId, "parent_file_id": sid}
            requests_post = requests.post('https://api.aliyundrive.com/v2/file/list',
                                          data=json.dumps(requests_data),
                                          headers=self.headers,
                                          verify=False).json()
            ret = []
            for item in requests_post['items']:
                obj = FileAttr()
                obj.isfile = item['type'] != 'folder'
                obj.name = item['name']
                obj.path = remotePath + '/' + item['name']
                obj.uid = item['file_id']
                obj.size = item['size'] if 'size' in item else 0
                if not obj.isfile:
                    self.__updatePathId__(obj.path, obj.uid)

                ret.append(obj)
            return ret

        except:
            return []

    def __mkdir__(self, folderName, parentFolderId='root') -> (bool, str):
        folderName = folderName.strip('/')
        try:
            requests_data = {
                "drive_id": self.driveId,
                "parent_file_id": parentFolderId,
                "name": folderName,
                "check_name_mode": "refuse",
                "type": "folder"
            }
            post_json = requests.post('https://api.aliyundrive.com/v2/file/create',
                                      data=json.dumps(requests_data),
                                      headers=self.headers,
                                      verify=False).json()
            return True, post_json.get('file_id')
        except:
            return False, ''

    def __formatRemotePath__(self, remotePath: str) -> str:
        remotePath = remotePath.rstrip('/')
        remotePath = remotePath.replace('//', '/')
        return remotePath

    def mkdirs(self, remotePath: str) -> bool:
        remotePath = remotePath.rstrip('/')
        if self.__getPathId__(remotePath):
            return True

        parent = ''
        paths = remotePath.split('/')
        for item in paths:
            key = parent + '/' + item
            key = key.replace('//', '/')
            if key not in self.pathIds:
                check, keyId = self.__mkdir__(item, self.pathIds[parent])
                if not check:
                    return False
                self.__updatePathId__(key, keyId)
            parent = key

        return True

    def uploadLink(self, localFilePath: str, remoteFilePath: str) -> AliUploadLink:
        filesize = os.path.getsize(localFilePath)
        filename = os.path.basename(localFilePath)
        hashcode = aigpy.file.getHash(localFilePath)
        if filesize <= 0:
            return None

        remotePath = aigpy.path.getDirName(remoteFilePath)
        remotePath = self.__formatRemotePath__(remotePath)
        if not self.mkdirs(remotePath):
            return None

        try:
            part_info_list = []
            for i in range(0, math.ceil(filesize / self.chunkSize)):
                part_info_list.append({'part_number': i + 1})

            requests_data = {
                "drive_id": self.driveId,
                "part_info_list": part_info_list,
                "parent_file_id": self.__getPathId__(remotePath),
                "name": filename,
                "type": "file",
                "check_name_mode": "refuse",
                "size": filesize,
                "content_hash": hashcode,
                "content_hash_name": 'sha1'
            }

            requests_post_json = requests.post('https://api.aliyundrive.com/v2/file/create',
                                               data=json.dumps(requests_data),
                                               headers=self.headers,
                                               verify=False).json()

            ret = AliUploadLink(requests_post_json, localFilePath, remoteFilePath)
            return ret
        except Exception as e:
            return None

    def __getXmlValue__(self, xml_string, tag_name):
        DOMTree = parseString(xml_string)
        DOMTree = DOMTree.documentElement
        tag = DOMTree.getElementsByTagName(tag_name)
        if len(tag) > 0:
            for node in tag[0].childNodes:
                if node.nodeType == node.TEXT_NODE:
                    return node.data
        return False

    # def __getUploadUrl__(self, fileId, filesize, uploadId):
    #     try:
    #         part_info_list = []
    #         for i in range(0, math.ceil(filesize / self.chunk_size)):
    #             part_info_list.append({'part_number': i + 1})

    #         requests_data = {
    #             "drive_id": self.driveId,
    #             "file_id": fileId,
    #             "part_info_list": part_info_list,
    #             "upload_id": uploadId,
    #         }
    #         requests_post = requests.post('https://api.aliyundrive.com/v2/file/get_upload_url',
    #                                       data=json.dumps(requests_data),
    #                                       headers=self.headers,
    #                                       verify=False
    #                                       )
    #         requests_post_json = requests_post.json()
    #         return requests_post_json.get('part_info_list')
    #     except:
    #         return ''

    def uploadFile(self, link: AliUploadLink, showProgress: bool = True) -> bool:
        if not link:
            return False
        if not link.needUpload:
            return True

        try:
            totalSize = os.path.getsize(link.localFilePath)
            progress = ProgressTool(totalSize, 15) if showProgress else None
            index = 0
            curcount = 0
            with open(link.localFilePath, 'rb') as f:
                while True:
                    chunk = f.read(self.chunkSize)

                    res = requests.put(url=link.list[index]['upload_url'],
                                       data=chunk,
                                       verify=False,
                                       timeout=None)

                    if 400 <= res.status_code < 600:
                        # message = self.__getXmlValue__(res.text, 'Message')
                        # if message == 'Request has expired.':
                        #     part_upload_url_list = self.__getUploadUrl__(file_id, filesize, upload_id)
                        #     if part_upload_url_list == '':
                        #         return False
                        #     continue
                        code = self.__getXmlValue__(res.text, 'Code')
                        if code == 'PartAlreadyExist':
                            pass
                        else:
                            res.raise_for_status()
                            return False
                    index += 1
                    curcount += len(chunk)
                    if progress:
                        progress.setCurCount(curcount)
                    if curcount >= totalSize:
                        break

            complete_data = {"drive_id": self.driveId,
                             "file_id": link.fileId,
                             "upload_id": link.uploadId
                             }
            requests_post_json = requests.post('https://api.aliyundrive.com/v2/file/complete',
                                               json.dumps(complete_data),
                                               headers=self.headers,
                                               verify=False).json()
            if 'file_id' in requests_post_json:
                return True
        except Exception as e:
            return False


class CheckFileExistCache(object):
    def __init__(self):
        self.fileMap = {}
        self.path = ''

    def setMap(self, path: str, fileAttrs: [FileAttr]):
        self.fileMap = {}
        for item in fileAttrs:
            if item.isfile:
                self.fileMap[item.name] = item
        self.path = path

    def getItem(self, name):
        if name in self.fileMap:
            return self.fileMap[name]
        return None


class AliPlat(PlatformImp):
    def __init__(self):
        super().__init__()
        self.checkFileExistCache = CheckFileExistCache()

    def list(self, remotePath: str, includeSubDir: bool = False) -> List[FileAttr]:
        array = []
        res = self.key.list(remotePath)
        for item in res:
            array.append(item)
            if includeSubDir and not item.isfile:
                subarr = self.list(item.path, includeSubDir)
                array.extend(subarr)
        return array

    def downloadFile(self, fileAttr: FileAttr, localFilePath: str) -> bool:
        return False

    def uploadFile(self, localFilePath: str, remoteFilePath: str) -> bool:
        link = self.key.uploadLink(localFilePath, remoteFilePath)
        if not link:
            return False
        check = self.key.uploadFile(link)
        return check

    def downloadLink(self, remoteFilePath: str):
        return None

    def uploadLink(self, localFilePath: str, remoteFilePath: str):
        link = self.key.uploadLink(localFilePath, remoteFilePath)
        return link

    def isFileExist(self, remoteFilePath: str) -> bool:
        path = aigpy.path.getDirName(remoteFilePath)
        name = aigpy.path.getFileName(remoteFilePath)

        if path != self.checkFileExistCache.path:
            array = self.key.list(path)
            self.checkFileExistCache.setMap(path, array)

        return self.checkFileExistCache.getItem(name) is not None



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  trans.py
@Date    :  2021/7/27
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import os

import aigpy

from b2a import AliPlat, BdyPlat
from b2a.platformImp import FileAttr

_DOWNLOAD_PATH = './b2a/download/'
aigpy.path.mkdirs(_DOWNLOAD_PATH)


class Trans(object):
    def __init__(self, aliplat: AliPlat, bdyPlat: BdyPlat):
        self._aliplat = aliplat
        self._bdyplat = bdyPlat
        self.successCnt = 0
        self.errCnt = 0
        self.skipCnt = 0
        self.index = 0
        self.baseFromPath = ''
        self.baseToPath = ''

    def clearCnt(self):
        self.successCnt = 0
        self.errCnt = 0
        self.skipCnt = 0
        self.index = 0

    def moveFile(self, item: FileAttr):
        self.index += 1

        uploadFilePath = self.baseToPath + item.path[len(self.baseFromPath):]
        if self._aliplat.isFileExist(uploadFilePath):
            self.skipCnt += 1
            aigpy.cmd.printInfo(f"[{self.index}] 跳过文件: {item.path}")
            return True

        aigpy.cmd.printInfo(f"[{self.index}] 迁移文件: {item.path}")
        localFilePath = _DOWNLOAD_PATH + item.path
        if aigpy.file.getSize(localFilePath) <= 0:
            tmpFile = localFilePath + ".tmp"
            check = self._bdyplat.downloadFile(item, tmpFile)
            if not check:
                aigpy.path.remove(tmpFile)
                aigpy.cmd.printErr("[错误] 下载失败!")
                self.errCnt += 1
                return False
            else:
                os.rename(tmpFile, localFilePath)

        check = self._aliplat.uploadFile(localFilePath, uploadFilePath)
        if not check:
            aigpy.cmd.printErr("[错误] 上传失败!")
            self.errCnt += 1
        else:
            self.successCnt += 1

        aigpy.path.remove(localFilePath)

    def __movePath__(self, fromPath: str):
        array = self._bdyplat.list(fromPath)
        for item in array:
            if item.isfile:
                self.moveFile(item)
        for item in array:
            if not item.isfile:
                self.__movePath__(item.path)

    def setPath(self, fromPath: str, toPath: str):
        self.baseFromPath = fromPath
        self.baseToPath = toPath

    def start(self):
        self.clearCnt()
        self.__movePath__(self.baseFromPath)
        aigpy.path.remove(_DOWNLOAD_PATH)
        aigpy.cmd.printInfo(f"迁移文件：{self.successCnt}；失败：{self.errCnt}；跳过：{self.skipCnt}")



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  download.py
@Date    :  2021/07/23
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""
import _thread
import threading

import aigpy
from PyQt5.QtWidgets import QWidget, QTableView, QLabel, QGridLayout

import b2a
from b2a.platformImp import FileAttr


class Task(object):
    def __init__(self):
        self.index = 0
        self.fileAttr = None
        self.status = '等待'
        self.isError = False


class TaskManager(object):
    def __init__(self):
        self.tasks = []
        self.waitTasks = []

    def clear(self):
        pass

    def addTask(self, item: FileAttr):
        task = Task()
        task.fileAttr = item
        task.index = len(self.tasks)
        self.tasks.append(task)
        self.waitTasks.append(task)

    def getWaitTask(self) -> Task:
        pass


class LoadThread(threading.Thread):
    def __init__(self, view):
        threading.Thread.__init__(self)
        self.view = view

    def __load__(self, array):
        for item in array:
            if item.isfile:
                self.view.taskManager.addTask(item)
        for item in array:
            if not item.isfile:
                subArray = b2a.bdyplat.list(item.path)
                self.__load__(subArray)

    def run(self):
        print("开始线程：LoadThread")
        self.__load__(self.view.bdyFileAttrs)
        print("退出线程：LoadThread")


class DealThread(threading.Thread):
    def __init__(self, view):
        threading.Thread.__init__(self)
        self.view = view

    def run(self):
        print("开始线程：DealThread")
        while True:
            task = self.view.taskManager.getWaitTask()
            item = task.fileAttr

            localFilePath = b2a.getDownloadPath() + item.path
            uploadFilePath = self.view.aliPath + '/' + item.path[len(self.view.bdyPath):]
            if b2a.aliplat.isFileExist(uploadFilePath):
                b2a.asyncCount.skip += 1
                aigpy.cmd.printInfo(f"[{b2a.asyncCount.index}] 跳过文件: {item.path}")
                continue

            aigpy.cmd.printInfo(f"[{b2a.asyncCount.index}] 迁移文件: {item.path}")
            if aigpy.file.getSize(localFilePath) <= 0:
                check = b2a.bdyplat.downloadFile(item.path, localFilePath)
                if not check:
                    aigpy.cmd.printErr("[错误] 下载失败!")
                    b2a.asyncCount.err += 1
                    continue

            check = b2a.aliplat.uploadFile(localFilePath, uploadFilePath)
            if not check:
                aigpy.cmd.printErr("[错误] 上传失败!")
                b2a.asyncCount.err += 1
            else:
                b2a.asyncCount.success += 1

            aigpy.path.remove(localFilePath)
        print("退出线程：DealThread")


class DownloadView(QWidget):

    def __init__(self):
        super().__init__()
        self.taskManager = TaskManager()
        self.bdyPath = ''
        self.aliPath = ''
        self.bdyFileAttrs = []
        self.__initView__()

    def __initView__(self):
        self.info = QLabel()
        self.status = QLabel()
        self.tableView = QTableView()

        self.layout = QGridLayout(self)
        self.layout.addWidget(self.info, 0, 0)
        self.layout.addWidget(self.tableView, 1, 0)
        self.layout.addWidget(self.status, 2, 0)

        self.hide()

    def updateInfo(self):
        self.info.setText(f"成功：{b2a.asyncCount.success}；失败：{b2a.asyncCount.err}；跳过：{b2a.asyncCount.skip}")

    def start(self, array: [FileAttr], bdyPath: str, aliPath: str):
        self.taskManager.clear()
        self.bdyPath = bdyPath
        self.bdyFileAttrs = array
        self.aliPath = aliPath

        loadThread = LoadThread(self)
        dealThread = DealThread(self)
        loadThread.start()
        dealThread.start()

        self.show()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  __init__.py
@Date    :  2021/06/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import getopt
import sys

import aigpy
from prettytable import prettytable

from b2a.aliplat import AliPlat, AliKey
from b2a.bdyplat import BdyPlat, BdyKey
from b2a.config import B2aConfig
from b2a.platformImp import PlatformImp
from b2a.trans import Trans

__LOGO__ = '''
         /$$$$$$$   /$$$$$$   /$$$$$$         
        | $$__  $$ /$$__  $$ /$$__  $$        
        | $$  \ $$|__/  \ $$| $$  \ $$        
 /$$$$$$| $$$$$$$   /$$$$$$/| $$$$$$$$ /$$$$$$
|______/| $$__  $$ /$$____/ | $$__  $$|______/
        | $$  \ $$| $$      | $$  | $$        
        | $$$$$$$/| $$$$$$$$| $$  | $$        
        |_______/ |________/|__/  |__/        

  https://github.com/yaronzz/BaiduYunToAliYun 
'''
VERSION = '2021.7.30.5'

aliplat = AliPlat()
bdyplat = BdyPlat()
config = B2aConfig()
trans = Trans(aliplat, bdyplat)


def loginAli(token: str) -> bool:
    key = AliKey()
    if not key.login(token):
        aigpy.cmd.printErr("登录阿里云失败!")
        return False
    aliplat.setKey(key)
    config.aliKey = key.refreshToken
    if not config.save():
        aigpy.cmd.printErr("保存登录信息文件失败!")
    return True


def loginBdy(cookies: str) -> bool:
    key = BdyKey()
    if not key.login(cookies):
        aigpy.cmd.printErr("登录百度云失败!")
        return False
    bdyplat.setKey(key)
    config.bdyKey = key.cookies
    if not config.save():
        aigpy.cmd.printErr("保存登录信息文件失败!")
    return True


def listPath(plat: PlatformImp, remotePath: str):
    array = plat.list(remotePath)
    aigpy.cmd.printInfo(f"目录列表项共有：{len(array)}项")
    for item in array:
        print(item.path)


def isLogin():
    if not aliplat.hasKey():
        aigpy.cmd.printErr("请先登录阿里云！")
        return False
    if not bdyplat.hasKey():
        aigpy.cmd.printErr("请先登录百度云！")
        return False
    return True


def asyncPath(bdyFromPath: str, aliToPath: str):
    if not isLogin():
        return
    trans.clearCnt()
    trans.setPath(bdyFromPath, aliToPath)
    trans.start()


def printChoices():
    print("====================================================")
    tb = prettytable.PrettyTable()
    tb.field_names = ["功能", "选项"]
    tb.align = 'l'
    tb.set_style(prettytable.PLAIN_COLUMNS)
    tb.add_row([aigpy.cmd.green("输入" + " '0':"), "退出"])
    tb.add_row([aigpy.cmd.green("输入" + " '1':"), "登录阿里云"])
    tb.add_row([aigpy.cmd.green("输入" + " '2':"), "登录百度云"])
    tb.add_row([aigpy.cmd.green("输入" + " '3':"), "显示阿里云目录"])
    tb.add_row([aigpy.cmd.green("输入" + " '4':"), "显示百度云目录"])
    tb.add_row([aigpy.cmd.green("输入" + " '5':"), "文件迁移"])
    print(tb)
    print("====================================================")


def printLogo():
    string = __LOGO__ + '\n               v' + VERSION
    print(string)


def printNewVersion():
    version = aigpy.pipHelper.getLastVersion('b2a')
    if aigpy.system.cmpVersion(version, VERSION) > 0:
        aigpy.cmd.printInfo("发现新版本：" + version)


def printUsage():
    print("=============B2A HELP==============")
    tb = prettytable.PrettyTable()
    tb.field_names = ["功能", "描述"]
    tb.align = 'l'
    tb.add_row(["-h or --help", "显示帮助"])
    tb.add_row(["-v or --version", "显示版本"])
    tb.add_row(["-a or --ali", "登录阿里云，参数为refresh_token"])
    tb.add_row(["-b or --bdy", "登录百度云，参数为cookies"])
    tb.add_row(["-f or --from", "待迁移的百度云目录"])
    tb.add_row(["-t or --to", "要存放的阿里云目录"])
    tb.add_row(["--alist", "显示阿里云目录"])
    tb.add_row(["--blist", "显示百度云目录"])
    print(tb)


def mainCommand():
    if config.aliKey:
        loginAli(config.aliKey)
    if config.bdyKey:
        loginBdy(config.bdyKey)

    try:
        opts, args = getopt.getopt(sys.argv[1:],
                                   "hva:b:f:t:",
                                   ["help", "version", "ali=", "bdy=", "from=", "to=", "alist=", "blist="])
    except getopt.GetoptError as errmsg:
        aigpy.cmd.printErr("输入参数错误!")
        printUsage()
        return

    bdyPath = ''
    aliPath = ''
    for opt, val in opts:
        if opt in ('-h', '--help'):
            printUsage()
            return
        if opt in ('-v', '--version'):
            printLogo()
            return
        if opt in ('-a', '--ali'):
            if loginAli(val):
                aigpy.cmd.printInfo("登录阿里云成功!")
            continue
        if opt in ('-b', '--bdy'):
            if loginBdy(val):
                aigpy.cmd.printInfo("登录百度云成功!")
            continue
        if opt in ('-f', '--from'):
            bdyPath = val
            if aliPath == '':
                aliPath = val
            continue
        if opt in ('-t', '--to'):
            aliPath = val
            continue
        if opt in ('--alist'):
            listPath(aliplat, val)
            continue
        if opt in ('--blist'):
            listPath(bdyplat, val)
            continue

    if aliPath == '' or bdyPath == '':
        return

    aigpy.cmd.printInfo(f"====迁移百度云[{bdyPath}]到阿里云[{aliPath}]====")
    asyncPath(bdyPath, aliPath)


def main():
    if len(sys.argv) > 1:
        mainCommand()
        return

    printLogo()
    printNewVersion()

    if config.aliKey and loginAli(config.aliKey):
        aigpy.cmd.printInfo("登录阿里云成功!")
    if config.bdyKey and loginBdy(config.bdyKey):
        aigpy.cmd.printInfo("登录百度云成功!")

    while True:
        printChoices()
        choice = aigpy.cmd.inputInt(aigpy.cmd.yellow("选项:"), 0)
        if choice == 0:
            return
        elif choice == 1:
            para = input(aigpy.cmd.yellow("请输入refresh_token:"))
            if loginAli(para):
                aigpy.cmd.printInfo("登录阿里云成功!")
        elif choice == 2:
            para = input(aigpy.cmd.yellow("请输入cookies:"))
            if loginBdy(para):
                aigpy.cmd.printInfo("登录百度云成功!")
        elif choice == 3:
            para = input(aigpy.cmd.yellow("请输入路径:"))
            listPath(aliplat, para)
        elif choice == 4:
            para = input(aigpy.cmd.yellow("请输入路径:"))
            listPath(bdyplat, para)
        elif choice == 5:
            fromPath = input(aigpy.cmd.yellow("请输入百度云路径:"))
            toPath = input(aigpy.cmd.yellow("请输入阿里云路径:"))
            asyncPath(fromPath, toPath)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  downloader.py
@Date    :  2021/7/29
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import os
from concurrent.futures import ThreadPoolExecutor, wait
from threading import Lock

import aigpy.cmdHelper
from aigpy.convertHelper import MemoryUnit, convertMemoryUnitAuto
from aigpy.progressHelper import ProgressTool
from requests import get, head
from tqdm import tqdm


class Downloader(object):
    def __init__(self, url, headers, filePath, size, threadNum=6):
        self.url = url
        self.threadNum = threadNum
        self.filePath = filePath
        self.size = size

        self._fp = None
        self._error = False
        self._headers = headers
        self._blockSize = 1 * 1024 * 1024
        self._lock = Lock()
        self._bar = tqdm(total=self.size, desc="下载中", unit_scale=True)

    def __getSize__(self, headers):
        r = head(self.url, headers=headers)
        while r.status_code == 302:
            url = r.headers['Location']
            r = head(url, headers=headers)
        if r.status_code == 200:
            return int(r.headers['Content-Length'])
        return 0

    def __createFile__(self):
        try:
            fp = open(self.filePath, "wb")
            fp.truncate(self.size)
            fp.close()
            return True
        except Exception as e:
            aigpy.cmd.printErr("创建文件失败：" + str(e))
            return False

    def down(self, start, end):
        try:
            if self._error:
                return False

            headers = {'Range': 'bytes={}-{}'.format(start, end)}
            r = get(self.url, headers=headers.update(self._headers), stream=True)
            r.raise_for_status()

            self._lock.acquire()
            if not self._error:
                self._fp.seek(start)
                self._fp.write(r.content)
                self._bar.update(end - start)
            self._lock.release()
            return True
        except Exception as e:
            aigpy.cmd.printErr("下载文件块失败：" + str(e))
            self._error = True
            return False

    def run(self):
        try:
            if not self.__createFile__():
                return False

            self._fp = open(self.filePath, "rb+")

            num = self.size // self._blockSize
            pool = ThreadPoolExecutor(max_workers=self.threadNum)
            futures = []
            for i in range(num):
                start = self._blockSize * i
                # 最后一块
                if i == num - 1:
                    end = self.size
                else:
                    end = start + self._blockSize - 1
                futures.append(pool.submit(self.down, start, end))
            wait(futures)
            self._bar.close()
            self._fp.close()
            return ~self._error
        except Exception as e:
            aigpy.cmdHelper.printErr("下载失败：" + str(e))
            self._error = True
            return False



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  bdyplat.py
@Date    :  2021/06/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import aigpy
from baidupcs_py.baidupcs import BaiduPCSApi
from requests import head
from tqdm import *

from b2a.downloader import Downloader
from b2a.platformImp import *


class BdyKey(object):
    def __init__(self):
        super().__init__()
        self.api = None
        self.cookies = ''

    def login(self, cookiesStr: str) -> bool:
        try:
            cookies = dict()
            array = cookiesStr.split(';')
            for item in array:
                values = item.split('=')
                cookies[values[0].strip(' ')] = values[1]
            self.api = BaiduPCSApi(bduss=cookies['BDUSS'], cookies=cookies)
            self.cookies = cookiesStr
            return True
        except:
            return False


class BdyPlat(PlatformImp):
    def __init__(self):
        super().__init__()

    def list(self, remotePath: str, includeSubDir: bool = False) -> List[FileAttr]:
        array = []
        if len(remotePath) <= 0:
            remotePath = '/'
        res = self.key.api.list(remotePath)
        for item in res:
            obj = FileAttr()
            obj.isfile = item.is_file
            obj.name = aigpy.path.getFileName(item.path)
            obj.path = item.path
            obj.size = item.size
            obj.uid = item.fs_id
            array.append(obj)

            if includeSubDir and item.is_dir:
                subarr = self.list(item.path, includeSubDir)
                array.extend(subarr)
        return array

    def downloadFile(self, fileAttr: FileAttr, localFilePath: str) -> bool:
        path = aigpy.path.getDirName(localFilePath)
        name = aigpy.path.getFileName(localFilePath)
        check = aigpy.path.mkdirs(path)

        # stream = self.key.api.file_stream(fileAttr.path)
        # if not stream:
        #     return False
        #
        # curSize = 0
        # totalSize = len(stream)
        # with open(localFilePath, 'wb+') as f:
        #     with tqdm.wrapattr(stream, "read", desc='下载中', miniters=1, total=totalSize, ascii=True) as fs:
        #         while True:
        #             data = fs.read(2097152)
        #             f.write(data)
        #             curSize += len(data)
        #             if curSize >= totalSize:
        #                 break
        #
        # stream.close()
        # return True

        headers = {
            "Cookie ": "; ".join(
                [f"{k}={v if v is not None else ''}" for k, v in self.key.api.cookies.items()]
            ),
            "User-Agent": "netdisk;2.2.51.6;netdisk;10.0.63;PC;android-android",
            "Connection": "Keep-Alive",
        }

        link = self.key.api.download_link(fileAttr.path)
        if not link or len(link) <= 0:
            return False

        dl = Downloader(link, headers, localFilePath, fileAttr.size)
        check = dl.run()
        return check

    def uploadFile(self, localFilePath: str, remoteFilePath: str) -> bool:
        return False

    def downloadLink(self, remoteFilePath: str):
        link = self.key.api.download_link(remoteFilePath)
        return link

    def uploadLink(self, localFilePath: str, remoteFilePath: str):
        return None

    def isFileExist(self, remoteFilePath: str) -> bool:
        return self.key.api.is_file(remoteFilePath)





#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  config.py
@Date    :  2021/7/27
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import json
import os

import aigpy

_BASE_PATH = os.path.expanduser('~') + '/b2a/'
_CONFIG_FILE_PATH = f"{_BASE_PATH}auth.json"
_ALI_KEY = "ali-refresh_token"
_BDY_KEY = "bdy-cookies"


class B2aConfig(object):
    def __init__(self):
        aigpy.path.mkdirs(_BASE_PATH)
        self.aliKey = ""
        self.bdyKey = ""
        self.load()

    def load(self):
        authJson = aigpy.file.getJson(_CONFIG_FILE_PATH)
        self.aliKey = authJson.get(_ALI_KEY)
        self.bdyKey = authJson.get(_BDY_KEY)

    def save(self) -> bool:
        content = dict()
        content[_ALI_KEY] = self.aliKey
        content[_BDY_KEY] = self.bdyKey
        if not aigpy.file.write(_CONFIG_FILE_PATH, json.dumps(content), 'w+'):
            return False
        return True



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  __init__.py
@Date    :  2021/07/23
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""
import sys

from PyQt5.QtWidgets import QApplication, QMessageBox, QPushButton

import b2a
from b2a_gui.main import MainView


def login() -> bool:
    if not b2a.loginAli(b2a.config.aliKey):
        return False
    if not b2a.loginBdy(b2a.config.bdyKey):
        return False
    return True


if __name__ == '__main__':
    app = QApplication(sys.argv)
    if not login():
        qmb = QMessageBox()
        qmb.setWindowTitle('错误')
        qmb.setText('<h2>登录失败</h2>')
        qmb.setInformativeText('请先正确登录阿里云与百度云！')
        qmb.addButton(QPushButton('确定', qmb), QMessageBox.YesRole)
        qmb.open()
    else:
        ex = MainView()
    sys.exit(app.exec_())



from setuptools import setup, find_packages
from b2a.__init__ import VERSION

setup(
    name='b2a',
    version=VERSION,
    license="MIT",
    description="迁移百度云到阿里云",

    author='Yaronzz',
    author_email="yaronhuang@foxmail.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=["aigpy", "requests", "tqdm", "prettytable", "Cython", "BaiduPCS-Py"],
    entry_points={'console_scripts': ['b2a = b2a:main', ]}
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  main.py
@Date    :  2021/07/23
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtWidgets import QWidget, QPushButton, QLineEdit, QInputDialog, QLabel, QTableView, QGridLayout, \
    QAbstractItemView, QHeaderView

import b2a


class MainView(QWidget):

    def __init__(self):
        super().__init__()
        self._fileAttrs = []
        self.__initView__()

    def __initView__(self):
        self.label0 = QLabel("从bdy:")
        self.label1 = QLabel("到aly:")

        self.edit0 = QLineEdit("/")
        self.edit0.setEnabled(False)
        self.edit1 = QLineEdit("/")

        self.btn0 = QPushButton('UP')
        self.btn0.clicked.connect(self.__upSlot__)
        self.btn1 = QPushButton('RUN')
        self.btn1.clicked.connect(self.__runSlot__)

        self.tableView = QTableView()
        self.tableView.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.tableView.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.tableView.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        self.tableView.doubleClicked.connect(self.__inSlot__)

        self.layout = QGridLayout(self)
        self.layout.addWidget(self.label0, 0, 0)
        self.layout.addWidget(self.edit0, 0, 1)
        self.layout.addWidget(self.btn0, 0, 2)
        self.layout.addWidget(self.tableView, 1, 0, 1, 3)
        self.layout.addWidget(self.label1, 2, 0)
        self.layout.addWidget(self.edit1, 2, 1)
        self.layout.addWidget(self.btn1, 2, 2)

        self.setWindowTitle('B2A')
        self.setMinimumSize(400,400)
        self.show()

        self.listPath('/')

    def __upSlot__(self):
        path = self.edit0.text()
        if path == '/':
            return
        array = path.rstrip('/').split('/')
        newPath = '/'.join(array[0:-1])
        self.listPath(newPath)

    def __runSlot__(self):
        pass

    def __inSlot__(self):
        index = self.tableView.currentIndex()
        if not index.isValid():
            return
        item = self._fileAttrs[index.row()]
        if item.isfile:
            return
        self.listPath(item.path)

    def listPath(self, path: str):
        self._fileAttrs = b2a.bdyplat.list(path)
        model = QStandardItemModel()
        for index, item in enumerate(self._fileAttrs):
            model.setItem(index, 0, QStandardItem(item.name))
            model.setItem(index, 1, QStandardItem("文件" if item.isfile else "目录"))
        self.tableView.setModel(model)
        self.edit0.setText(path)
        self.edit1.setText(path)




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  platformImp.py
@Date    :  2021/06/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import abc
from typing import List


class FileAttr(object):
    def __init__(self, isfile=False, name='', size=0, path='', uid=''):
        self.isfile = isfile
        self.name = name
        self.size = int(size)
        self.path = path
        self.uid = uid


class PlatformImp(metaclass=abc.ABCMeta):
    def __init__(self):
        self.key = None

    def hasKey(self) -> bool:
        return self.key is not None

    def setKey(self, key):
        self.key = key

    @abc.abstractmethod
    def list(self, remotePath: str) -> List[FileAttr]:
        pass

    @abc.abstractmethod
    def isFileExist(self, remoteFilePath: str) -> bool:
        pass

    @abc.abstractmethod
    def downloadFile(self, fileAttr: FileAttr, localFilePath: str) -> bool:
        pass

    @abc.abstractmethod
    def uploadFile(self, localFilePath: str, remoteFilePath: str) -> bool:
        pass

    @abc.abstractmethod
    def downloadLink(self, remoteFilePath: str):
        pass
    
    @abc.abstractmethod
    def uploadLink(self, localFilePath: str, remoteFilePath: str):
        pass
